@@ -44,14 +44,16 @@ window (``(0.5, 0.5)``) and then clicks the left mouse button once
     action = Mouse("(0.5, 0.5), left")
     action.execute()
 
-The line below moves the mouse cursor to 100 pixels left of the desktop's
-right edge and 250 pixels down from its top edge (``[-100, 250]``), and
-then double clicks the right mouse button (``right:2``)::
+The line below moves the mouse cursor to 100 pixels left of the primary
+monitor's left edge (if possible) and 250 pixels down from its top edge
+(``[-100, 250]``), and then double clicks the right mouse button
+(``right:2``)::
 
     # Square brackets ("[...]") give desktop-relative locations.
     # Integer locations ("1", "100", etc.) denote numbers of pixels.
-    # Negative numbers ("-100") are counted from the right-edge or the
-    #  bottom-edge of the desktop or window.
+    # Negative numbers ("-100") are counted from the left-edge of the
+    #  primary monitor. They are used to access monitors above or to the
+    #  left of the primary monitor.
     Mouse("[-100, 250], right:2").execute()
 
 The following command drags the mouse from the top right corner of the
@@ -78,9 +80,10 @@ following possible formats:
 
 Mouse movement actions:
 
- - location is absolute on the entire desktop:
+ - move the cursor relative to the top-left corner of the desktop monitor
+   containing coordinates ``[0, 0]`` (i.e. the primary monitor):
    ``[`` *number* ``,`` *number* ``]``
- - location is relative to the foreground window:
+ - move the cursor relative to the foreground window:
    ``(`` *number* ``,`` *number* ``)``
  - move the cursor relative to its current position:
    ``<`` *pixels* ``,`` *pixels* ``>``
@@ -143,6 +146,25 @@ Mouse button-hold or button-release action:
    Specifies how long to pause *after* clicking the button; same as above.
 
 
+Mouse across platforms
+............................................................................
+
+Please note that there are some platforms which do not support emulating
+every mouse button listed above.  If an unsupported mouse button (*keyname*)
+is specified and the :class:`Mouse` action executed, an error is raised. For
+instance, scrolling the mouse wheel horizontally (e.g. *wheelleft*) is not,
+by default, a supported operation on X11::
+
+   ValueError: Unsupported scroll event: wheelleft
+
+Fortunately, this particular problem can be fixed by installing the *pynput*
+library::
+
+   pip install pynput
+
+On MacOS, however, Dragonfly cannot be used to scroll horizontally.
+
+
 Mouse class reference
 ............................................................................
 
@@ -151,15 +173,16 @@ Mouse class reference
 # pylint: disable=R0201
 # Suppress warnings about handler functions defined as Mouse methods.
 
-from .action_base       import DynStrActionBase, ActionError
-from ..windows.window   import Window
-
-from .mouse import (ButtonEvent, PauseEvent, MoveRelativeEvent,
-                    MoveScreenEvent, MoveWindowEvent, PLATFORM_BUTTON_FLAGS,
-                    PLATFORM_WHEEL_FLAGS)
+from dragonfly.actions.action_base import DynStrActionBase, ActionError
+from dragonfly.windows.window      import Window
+from dragonfly.actions.mouse       import (
+    ButtonEvent, PauseEvent, MoveRelativeEvent,
+    MoveScreenEvent, MoveWindowEvent, PLATFORM_BUTTON_FLAGS,
+    PLATFORM_WHEEL_FLAGS
+)
 
 # Imported for backwards-compatibility: these functions used to live here.
-from .mouse import get_cursor_position, set_cursor_position
+from dragonfly.actions.mouse import get_cursor_position, set_cursor_position
 
 
 class Mouse(DynStrActionBase):
@@ -215,8 +238,8 @@ class Mouse(DynStrActionBase):
     def _process_screen_position(self, spec, events):
         if not spec.startswith("[") or not spec.endswith("]"):
             return False
-        h_origin, h_value, v_origin, v_value = self._parse_position_pair(spec[1:-1])
-        event = MoveScreenEvent(h_origin, h_value, v_origin, v_value)
+        _, h_value, _, v_value = self._parse_position_pair(spec[1:-1])
+        event = MoveScreenEvent(True, h_value, True, v_value)
         events.append(event)
         return True
 
