@@ -31,7 +31,7 @@ from six.moves import queue, range
 import sounddevice
 import webrtcvad
 
-from ..base import EngineError
+from dragonfly.engines.base import EngineError
 
 _log = logging.getLogger("engine")
 
@@ -47,16 +47,31 @@ class MicAudio(object):
     BLOCK_SIZE_SAMPLES = int(SAMPLE_RATE / float(BLOCKS_PER_SECOND))  # Block size in number of samples
     BLOCK_DURATION_MS = int(1000 * BLOCK_SIZE_SAMPLES // SAMPLE_RATE)  # Block duration in milliseconds
 
-    def __init__(self, callback=None, buffer_s=0, flush_queue=True, start=True, input_device_index=None, self_threaded=None):
+    def __init__(self, callback=None, buffer_s=0, flush_queue=True, start=True, input_device=None, self_threaded=None, reconnect_callback=None):
         self.callback = callback if callback is not None else lambda in_data: self.buffer_queue.put(in_data, block=False)
-        self.flush_queue = flush_queue
-        self.input_device_index = int(input_device_index) if input_device_index is not None else None
+        self.flush_queue = bool(flush_queue)
+        self.input_device = input_device
         self.self_threaded = bool(self_threaded)
+        if reconnect_callback is not None and not callable(reconnect_callback):
+            _log.error("Invalid reconnect_callback not callable: %r", reconnect_callback)
+            reconnect_callback = None
+        self.reconnect_callback = reconnect_callback
 
         self.buffer_queue = queue.Queue(maxsize=(buffer_s * 1000 // self.BLOCK_DURATION_MS))
         self.stream = None
         self.thread = None
+        self.thread_cancelled = False
         self.device_info = None
+
+        try:
+            device_list = sounddevice.query_devices(device=self.input_device)
+            if not device_list:
+                raise EngineError("No audio devices found.")
+        except ValueError as e:
+            message = e.args[0]
+            message += "\nAvailable devices are:\n" + str(sounddevice.query_devices())
+            raise ValueError(message)
+
         self._connect(start=start)
 
     def _connect(self, start=None):
@@ -70,11 +85,12 @@ class MicAudio(object):
             dtype=self.FORMAT,
             blocksize=self.BLOCK_SIZE_SAMPLES,
             # latency=80,
-            device=self.input_device_index,
+            device=self.input_device,
             callback=proxy_callback if not self.self_threaded else None,
         )
 
         if self.self_threaded:
+            self.thread_cancelled = False
             self.thread = threading.Thread(target=self._reader_thread, args=(callback,))
             self.thread.daemon = True
             self.thread.start()
@@ -89,9 +105,8 @@ class MicAudio(object):
         self.device_info = device_info
 
     def _reader_thread(self, callback):
-        while self.stream and not self.stream.closed:
-            read_available = self.stream.read_available
-            if read_available >= self.stream.blocksize:
+        while not self.thread_cancelled and self.stream and not self.stream.closed:
+            if self.stream.active and self.stream.read_available >= self.stream.blocksize:
                 in_data, overflowed = self.stream.read(self.stream.blocksize)
                 # print('_reader_thread', read_available, len(in_data), overflowed, self.stream.blocksize)
                 if overflowed:
@@ -99,18 +114,27 @@ class MicAudio(object):
                 callback(bytes(in_data))  # Must copy data from temporary C buffer!
             else:
                 time.sleep(0.001)
+                
+    def _cancel_reader_thread(self):
+        self.thread_cancelled = True
+        if self.thread:
+            self.thread.join()
+            self.thread = None
 
     def destroy(self):
-        self.stream.close()
+        self._cancel_reader_thread()
+        if self.stream:
+            self.stream.close()
+            self.stream = None
 
     def reconnect(self):
         # FIXME: flapping
         old_device_info = self.device_info
+        self._cancel_reader_thread()
         self.stream.close()
-        if self.thread:
-            self.thread.join()
-            self.thread = None
         self._connect(start=True)
+        if self.reconnect_callback is not None:
+            self.reconnect_callback(self)
         if self.device_info != old_device_info:
             raise EngineError("Audio reconnect could not reconnect to the same device")
 
@@ -206,7 +230,7 @@ class VADAudio(MicAudio):
 
     def vad_collector(self, start_window_ms=150, start_padding_ms=100,
         end_window_ms=150, end_padding_ms=None, complex_end_window_ms=None,
-        ratio=0.8, blocks=None, nowait=False,
+        ratio=0.8, blocks=None, nowait=False, audio_auto_reconnect=False,
         ):
         """Generator/coroutine that yields series of consecutive audio blocks comprising each phrase, separated by yielding a single None.
             Determines voice activity by ratio of blocks in window_ms. Uses a buffer to include window_ms prior to being triggered.
@@ -241,12 +265,12 @@ class VADAudio(MicAudio):
             if block is False or block is None:
                 # Bad/empty block
                 num_empty_blocks += 1
-                if (num_empty_blocks >= audio_reconnect_threshold_blocks) and (time.time() - last_good_block_time >= audio_reconnect_threshold_time):
+                if audio_auto_reconnect and (num_empty_blocks >= audio_reconnect_threshold_blocks) and (time.time() - last_good_block_time >= audio_reconnect_threshold_time):
                     _log.warning("%s: no good block received recently, so reconnecting audio", self)
                     self.reconnect()
                     num_empty_blocks = 0
                     last_good_block_time = time.time()
-                in_complex_phrase = yield block
+                in_complex_phrase = yield False
 
             else:
                 # Good block
@@ -284,6 +308,10 @@ class VADAudio(MicAudio):
                         # print('*')
                         ring_buffer.clear()
 
+        if triggered:
+            # We were in a phrase, so we must terminate it (this may be abrupt!)
+            yield None
+
     def debug_print_simple(self):
         print("block_duration_ms=%s" % self.BLOCK_DURATION_MS)
         for block in self.iter(nowait=False):
@@ -411,10 +439,11 @@ class AudioStoreEntry(object):
 
 
 class WavAudio(object):
+    """ Class for mimicking normal microphone input, but from wav files. """
 
     @classmethod
-    def read_file(cls, filename):
-        """ Yields raw audio blocks from wav file. """
+    def read_file(cls, filename, realtime=False):
+        """ Yields raw audio blocks from wav file, terminated by a None element. """
         if not os.path.isfile(filename):
             raise IOError("'%s' is not a file. Please use a different file path.")
 
@@ -430,9 +459,22 @@ class WavAudio(object):
                 raise ValueError("WAV file '%s' should use sample rate %d, not "
                            "%d!" % (filename, MicAudio.SAMPLE_RATE, file.getframerate()))
 
+            next_time = time.time()
             for _ in range(0, int(file.getnframes() / MicAudio.BLOCK_SIZE_SAMPLES) + 1):
                 data = file.readframes(MicAudio.BLOCK_SIZE_SAMPLES)
                 if not data:
                     break
+                if realtime:
+                    time_behind = next_time - time.time()
+                    if time_behind > 0:
+                        time.sleep(time_behind)
+                    next_time += float(MicAudio.BLOCK_SIZE_SAMPLES) / MicAudio.SAMPLE_RATE
                 yield data
             yield None
+
+    @classmethod
+    def read_file_with_vad(cls, filename, realtime=False, **kwargs):
+        """ Yields raw audio blocks from wav file, after processing by VAD, terminated by a None element. """
+        vad_audio = VADAudio()
+        vad_audio_iter = vad_audio.vad_collector(blocks=cls.read_file(filename, realtime=realtime), **kwargs)
+        return vad_audio_iter
