@@ -24,30 +24,33 @@ Kaldi engine classes
 
 import collections, logging, os, sys, time
 
-from packaging.version import Version
-from six import string_types, print_, reraise
-from six.moves import zip
 import kaldi_active_grammar
-from kaldi_active_grammar       import KaldiError, KaldiRule
+from packaging.version     import Version
+from six                   import string_types, print_, reraise
+from six.moves             import zip
+from kaldi_active_grammar  import KaldiError, KaldiRule
 
-from ..base                     import (EngineBase,
-                                        EngineError,
-                                        MimicFailure,
-                                        DelegateTimerManager,
-                                        DelegateTimerManagerInterface,
-                                        DictationContainerBase,
-                                        GrammarWrapperBase)
-from .audio                     import MicAudio, VADAudio, AudioStore, WavAudio
-from .dictation                 import user_dictation_list, user_dictation_dictlist
-from .recobs                    import KaldiRecObsManager
-from .testing                   import debug_timer
-from dragonfly.grammar.state    import State
-from dragonfly.windows          import Window
+import dragonfly.engines
+from dragonfly.windows.window  import Window
+from dragonfly.engines.base    import (EngineBase,
+                                       EngineError,
+                                       MimicFailure,
+                                       DelegateTimerManager,
+                                       DelegateTimerManagerInterface,
+                                       DictationContainerBase,
+                                       GrammarWrapperBase)
+from dragonfly.engines.backend_kaldi.audio      import (MicAudio, VADAudio,
+                                                        AudioStore,
+                                                        WavAudio)
+from dragonfly.engines.backend_kaldi.dictation  import (user_dictation_list,
+                                                        user_dictation_dictlist)
+from dragonfly.engines.backend_kaldi.recobs     import KaldiRecObsManager
+from dragonfly.engines.backend_kaldi.testing    import debug_timer
 
 # Import the Kaldi compiler class. Suppress metaclass TypeErrors raised
 # during documentation builds caused by mocking KAG.
 try:
-    from .compiler                  import KaldiCompiler
+    from dragonfly.engines.backend_kaldi.compiler import KaldiCompiler
 except TypeError:
     if os.environ.get("SPHINX_BUILD_RUNNING"):
         KaldiCompiler = None
@@ -63,7 +66,8 @@ class KaldiEngine(EngineBase, DelegateTimerManagerInterface):
 
     _name = "kaldi"
     DictationContainer = DictationContainerBase
-    _required_kag_version = "2.1.0"
+    # NOTE: Remember to also update setup.py to the same version!
+    _required_kag_version = "3.1.0"
 
     #-----------------------------------------------------------------------
 
@@ -71,7 +75,8 @@ class KaldiEngine(EngineBase, DelegateTimerManagerInterface):
         audio_input_device=None, audio_self_threaded=True, audio_auto_reconnect=True, audio_reconnect_callback=None,
         retain_dir=None, retain_audio=None, retain_metadata=None, retain_approval_func=None,
         vad_aggressiveness=3, vad_padding_start_ms=150, vad_padding_end_ms=200, vad_complex_padding_end_ms=600,
-        auto_add_to_user_lexicon=True, lazy_compilation=True, invalidate_cache=False,
+        auto_add_to_user_lexicon=True, allow_online_pronunciations=False,
+        lazy_compilation=True, invalidate_cache=False,
         expected_error_rate_threshold=None,
         alternative_dictation=None,
         compiler_init_config=None, decoder_init_config=None,
@@ -129,6 +134,7 @@ class KaldiEngine(EngineBase, DelegateTimerManagerInterface):
             vad_padding_end_ms = int(vad_padding_end_ms),
             vad_complex_padding_end_ms = int(vad_complex_padding_end_ms),
             auto_add_to_user_lexicon = bool(auto_add_to_user_lexicon),
+            allow_online_pronunciations = bool(allow_online_pronunciations),
             lazy_compilation = bool(lazy_compilation),
             invalidate_cache = bool(invalidate_cache),
             expected_error_rate_threshold = float(expected_error_rate_threshold) if expected_error_rate_threshold is not None else None,
@@ -169,9 +175,10 @@ class KaldiEngine(EngineBase, DelegateTimerManagerInterface):
 
         self._compiler = KaldiCompiler(self._options['model_dir'], tmp_dir=self._options['tmp_dir'],
             auto_add_to_user_lexicon=self._options['auto_add_to_user_lexicon'],
+            allow_online_pronunciations=self._options['allow_online_pronunciations'],
             lazy_compilation=self._options['lazy_compilation'],
             alternative_dictation=self._options['alternative_dictation'],
-            **self._options['compiler_init_config'],
+            **self._options['compiler_init_config']
             )
         if self._options['invalidate_cache']:
             self._compiler.fst_cache.invalidate()
@@ -314,9 +321,7 @@ class KaldiEngine(EngineBase, DelegateTimerManagerInterface):
 
     def speak(self, text):
         """ Speak the given *text* using text-to-speech. """
-        # FIXME
-        self._log.warning("Text-to-speech is not implemented for this engine; printing text instead.")
-        print_(text)
+        dragonfly.engines.get_speaker().speak(text)
 
     def _get_language(self):
         return "en"
@@ -425,7 +430,7 @@ class KaldiEngine(EngineBase, DelegateTimerManagerInterface):
                             if kaldi_rule and is_acceptable_recognition:  # Don't store audio/metadata for bad recognitions
                                 self.audio_store.finalize(parsed_output,
                                     kaldi_rule.parent_grammar.name, kaldi_rule.parent_rule.name,
-                                    likelihood=expected_error_rate, has_dictation=kaldi_rule.has_dictation)
+                                    likelihood=expected_error_rate, has_dictation=recognition.has_dictation)
                             else:
                                 self.audio_store.cancel()
 
@@ -605,7 +610,7 @@ class KaldiEngine(EngineBase, DelegateTimerManagerInterface):
             if self._log.isEnabledFor(12):
                 try:
                     self._log.log(12, "Alignment (word,time,length): %s", self._decoder.get_word_align(output))
-                except KaldiError as e:
+                except KaldiError:
                     self._log.warning("Exception logging word alignment")
 
         else:
@@ -690,6 +695,20 @@ class GrammarWrapper(GrammarWrapperBase):
     def phrase_start_callback(self, executable, title, handle):
         self.grammar.process_begin(executable, title, handle)
 
+    def _decode_grammar_rules(self, state, words, results, *args):
+        rule = args[0]
+        state.initialize_decoding()
+        for result in rule.decode(state):
+            if state.finished():
+                root = state.build_parse_tree()
+                notify_args = (words, rule, root, results)
+                self.recobs_manager.notify_recognition(*notify_args)
+                with debug_timer(self.engine._log.debug, "rule execution time"):
+                    rule.process_recognition(root)
+                self.recobs_manager.notify_post_recognition(*notify_args)
+                return True
+        return False
+
     def recognition_callback(self, recognition):
         words = recognition.words
         rule = recognition.kaldi_rule.parent_rule
@@ -702,25 +721,8 @@ class GrammarWrapper(GrammarWrapperBase):
             words_rules = tuple((word, 0 if not is_dictation else 1)
                 for (word, is_dictation) in zip(words, words_are_dictation_mask))
 
-            # Attempt to parse the recognition
-            func = getattr(self.grammar, "process_recognition", None)
-            if func:
-                if not self._process_grammar_callback(func, words=words,
-                                                      results=recognition):
-                    # Return early if the method didn't return True or equiv.
-                    return
-
-            state = State(words_rules, rule_names, self.engine)
-            state.initialize_decoding()
-            for result in rule.decode(state):
-                if state.finished():
-                    root = state.build_parse_tree()
-                    notify_args = (words, rule, root, recognition)
-                    self.recobs_manager.notify_recognition(*notify_args)
-                    with debug_timer(self.engine._log.debug, "rule execution time"):
-                        rule.process_recognition(root)
-                    self.recobs_manager.notify_post_recognition(*notify_args)
-                    return
+            # Attempt to process the recognition.
+            if self.process_results(words_rules, rule_names, recognition, rule): return
 
         except Exception as e:
             self.engine._log.error("Grammar %s: exception: %s" % (self.grammar._name, e), exc_info=True)
